import pkg_resources
from importlib import import_module
from django.template import Library
from django.db.models import get_model
from ..utils import implements as base_implements


register = Library()


MESSAGE_LEVELS = {
    10: "debug",
    20: "info",
    25: "success",
    30: "warning",
    40: "error"
}


@register.inclusion_tag('djinn_core/snippets/css.html')
def list_plugin_css(static_url):

    css = []

    for entrypoint in pkg_resources.iter_entry_points(group="djinn.app",
                                                      name="css"):        
        css.extend(entrypoint.load()())

    def mkcss(link):

        props = {'media': 'screen'}

        if hasattr(link, "__iter__"):
            props['href'] = link[0]
            if len(link) > 1:
                props['media'] = link[1]            
        else:
            props['href'] = link

        return props

    css = map(mkcss, css)

    return {"plugin_css": css, "STATIC_URL": static_url}


@register.inclusion_tag('djinn_core/snippets/js.html')
def list_plugin_js(static_url):

    js = []

    for entrypoint in pkg_resources.iter_entry_points(group="djinn.app",
                                                      name="js"):
        js.extend(entrypoint.load()())

    return {"plugin_js": js, "STATIC_URL": static_url}


@register.filter(name="messageclass")
def messageclass(messages):
    
    """ Filter level from messages and return str representation """

    if len(messages):
        level = messages._loaded_messages[0].level
    else:
        level = 20

    return MESSAGE_LEVELS.get(level, "info")


@register.filter
def implements(instance, clazz):

    """ instance should be an object instance; clazz should be the
    full dotted name of the class, or an actual class, or a model specified
    as <app>.<model> """

    if isinstance(clazz, basestring):

        parts = clazz.split(".")

        if len(parts) == 2:
            clazz = get_model(*parts)
        else:
            clazz = getattr(import_module(".".join(parts[:-1])), parts[-1])

    return base_implements(instance, clazz)



from django.shortcuts import render_to_response
from django.template import RequestContext

import logging
log = logging.getLogger('djinn_core')

def djinn_server_error(request, template_name="500.html"):
    return render_to_response(template_name, {},
                        context_instance=RequestContext(request),)



from django.contrib.contenttypes.models import ContentType


# URN schema for objects
#
URN_SCHEMA = "urn:pu.in:%(object_app)s:%(object_ctype)s:%(object_id)s"


def _class_implements(clazz, superclazz, check_self=True):

    well_does_it = False

    if check_self and clazz == superclazz:
        return True
    elif isinstance(superclazz, basestring) and \
                    superclazz in map(lambda x: "%s.%s" %(x.__module__.split('.')[0], x.__name__), clazz.__bases__):
        well_does_it = True
    elif superclazz in  clazz.__bases__:
        well_does_it = True
    else:
        for base in clazz.__bases__:
            well_does_it = _class_implements(base, superclazz)
            if well_does_it:
                break

    return well_does_it


def implements(instance, clazz):

    """ Is it a bird? A plane? A clazz? """

    return _class_implements(instance.__class__, clazz)


def extends(clazz, otherclazz):

    """ Does the class extend the other one? """

    return _class_implements(clazz, otherclazz, check_self=False)


def object_to_urn(object):

    """ Create A URN for the given object """
    if not object:
        return ""

    app_label = getattr(object, "app_label", object._meta.app_label)
    ct_name = getattr(object, object.ct_name,
                      object.__class__.__name__.lower())

    return URN_SCHEMA % {'object_app': app_label,
                         'object_ctype': ct_name,
                         'object_id': object.id}


def urn_to_object(urn):

    """ Fetch the object for this URN. If not found, return None """

    parts = urn.split(":")

    ctype = ContentType.objects.get(app_label=parts[2], model=parts[3])

    try:
        return ctype.get_object_for_this_type(id=parts[4])
    except:
        return None



def get_object_by_ctype_id(ctype_id, _id, app_label=None):

    ctype = ContentType.objects.get(app_label=app_label, model=ctype_id)

    return ctype.get_object_for_this_type(id=_id)



import os, sys
import pkg_resources
from django.template.loaders.app_directories import Loader as BaseLoader
from django.utils.importlib import import_module
from django.core.exceptions import ImproperlyConfigured


fs_encoding = sys.getfilesystemencoding() or sys.getdefaultencoding()

app_template_dirs = []

for entrypoint in pkg_resources.iter_entry_points(group="djinn.skin"):
        
    app = entrypoint.module_name

    try:
        mod = import_module(app)
    except ImportError, e:
        raise ImproperlyConfigured('ImportError %s: %s' % (app, e.args[0]))

    template_dir = os.path.join(os.path.dirname(mod.__file__), 'templates')

    if os.path.isdir(template_dir):
        app_template_dirs.append(template_dir.decode(fs_encoding))


app_template_dirs = tuple(app_template_dirs)


class SkinTemplateLoader(BaseLoader):

    def load_template(self, template_name, template_dirs=None):

        return super(SkinTemplateLoader, self).load_template(
            template_name, template_dirs=app_template_dirs)


_loader = SkinTemplateLoader()



import os

from setuptools import setup, find_packages

here = os.path.abspath(os.path.dirname(__file__))
README = open(os.path.join(here, 'README.md')).read()
CHANGES = open(os.path.join(here, 'CHANGES.txt')).read()

requires = [
    'django'
    ]

setup(name='djinn_core',
      version="1.0.3",
      description='Djinn Intranet core',
      long_description=README + '\n\n' +  CHANGES,
      classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Framework :: Django",
        "Intended Audience :: Developers",
        "License :: Freely Distributable",
        "Programming Language :: Python",
        "Topic :: Internet :: WWW/HTTP :: Site Management",
        "Topic :: Software Development :: Libraries :: Application Frameworks"
        ],
      author='PythonUnited',
      author_email='info@pythonunited.com',
      license='beer-ware',
      url='https://github.com/PythonUnited/djinn_core',
      keywords='Djinn Django intranet',
      packages=find_packages(),
      include_package_data=True,
      zip_safe=False,
      install_requires = requires,
      tests_require= requires,
      test_suite="djinn_core",
      entry_points = """\
      [djinn.app]
      js=djinn_core:get_js
      """
      )



import os
import pkg_resources
from django.utils.datastructures import SortedDict
from django.contrib.staticfiles.finders import AppDirectoriesFinder
from django.contrib.staticfiles.storage import AppStaticStorage


class FlexiblePathStorage(AppStaticStorage):

    def __init__(self, app, *args, **kwargs):

        self.source_dir = kwargs.get("source_dir", "static")
        del kwargs['source_dir']

        super(FlexiblePathStorage, self).__init__(app, *args, **kwargs)

class PluginFilesFinder(AppDirectoriesFinder):

    def __init__(self, apps=None, *args, **kwargs):

        self.apps = []
        self.storages = SortedDict()

        for entrypoint in pkg_resources.iter_entry_points(group="djinn.app"):
        
            app = entrypoint.module_name

            if app in self.storages.keys():
                continue

            app_storage = self.storage_class(app)
            self.storages[app] = app_storage

        for entrypoint in pkg_resources.iter_entry_points(group="djinn.app",
                                                          name="statics"):
            path = entrypoint.load()()
            app = entrypoint.module_name
            
            self.storages[app] = FlexiblePathStorage(app, source_dir=path)

        for app in self.storages.keys():
            if os.path.isdir(self.storages[app].location):

                if app not in self.apps:
                    self.apps.append(app)
